@@ -1,28 +1,29 @@
 #!/usr/bin/python2.5
 
-# DialCentral - Front end for Google's Grand Central service.
-# Copyright (C) 2008  Mark Bergman bergman AT merctech DOT com
-# 
-# This library is free software; you can redistribute it and/or
-# modify it under the terms of the GNU Lesser General Public
-# License as published by the Free Software Foundation; either
-# version 2.1 of the License, or (at your option) any later version.
-# 
-# This library is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# Lesser General Public License for more details.
-# 
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+"""
+DialCentral - Front end for Google's Grand Central service.
+Copyright (C) 2008  Mark Bergman bergman AT merctech DOT com
+
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+"""
 
 from __future__ import with_statement
 
 import threading
 import time
 import warnings
-import traceback
 
 import gobject
 import gtk
@@ -307,10 +308,23 @@ class MergedAddressBook(object):
 
 class PhoneTypeSelector(object):
 
+	ACTION_CANCEL = "cancel"
+	ACTION_SELECT = "select"
+	ACTION_DIAL = "dial"
+	ACTION_SEND_SMS = "sms"
+
 	def __init__(self, widgetTree, gcBackend):
 		self._gcBackend = gcBackend
 		self._widgetTree = widgetTree
+
 		self._dialog = self._widgetTree.get_widget("phonetype_dialog")
+		self._smsDialog = SmsEntryDialog(self._widgetTree, self._gcBackend)
+
+		self._smsButton = self._widgetTree.get_widget("sms_button")
+		self._smsButton.connect("clicked", self._on_phonetype_send_sms)
+
+		self._dialButton = self._widgetTree.get_widget("dial_button")
+		self._dialButton.connect("clicked", self._on_phonetype_dial)
 
 		self._selectButton = self._widgetTree.get_widget("select_button")
 		self._selectButton.connect("clicked", self._on_phonetype_select)
@@ -321,44 +335,150 @@ class PhoneTypeSelector(object):
 		self._typemodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
 		self._typeviewselection = None
 
-		typeview = self._widgetTree.get_widget("phonetypes")
-		typeview.connect("row-activated", self._on_phonetype_select)
-		typeview.set_model(self._typemodel)
-		textrenderer = gtk.CellRendererText()
+		self._message = self._widgetTree.get_widget("phoneSelectionMessage")
+		self._typeview = self._widgetTree.get_widget("phonetypes")
+		self._typeview.connect("row-activated", self._on_phonetype_select)
+
+		self._action = self.ACTION_CANCEL
+
+	def run(self, contactDetails, message = ""):
+		self._action = self.ACTION_CANCEL
+		self._typemodel.clear()
+		self._typeview.set_model(self._typemodel)
 
 		# Add the column to the treeview
-		column = gtk.TreeViewColumn("Phone Numbers", textrenderer, text=1)
-		column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+		textrenderer = gtk.CellRendererText()
+		numberColumn = gtk.TreeViewColumn("Phone Numbers", textrenderer, text=0)
+		self._typeview.append_column(numberColumn)
 
-		typeview.append_column(column)
+		textrenderer = gtk.CellRendererText()
+		typeColumn = gtk.TreeViewColumn("Phone Type", textrenderer, text=1)
+		self._typeview.append_column(typeColumn)
 
-		self._typeviewselection = typeview.get_selection()
+		self._typeviewselection = self._typeview.get_selection()
 		self._typeviewselection.set_mode(gtk.SELECTION_SINGLE)
 
-	def run(self, contactDetails):
-		self._typemodel.clear()
-
 		for phoneType, phoneNumber in contactDetails:
-			self._typemodel.append((phoneNumber, "%s - %s" % (make_pretty(phoneNumber), phoneType)))
+			display = " - ".join((phoneNumber, phoneType))
+			display = phoneType
+			row = (phoneNumber, display)
+			self._typemodel.append(row)
+
+		self._typeviewselection.select_iter(self._typemodel.get_iter_first())
+		if message:
+			self._message.set_markup(message)
+			self._message.show()
+		else:
+			self._message.set_markup("")
+			self._message.hide()
 
 		userResponse = self._dialog.run()
 
 		if userResponse == gtk.RESPONSE_OK:
-			model, itr = self._typeviewselection.get_selected()
-			if itr:
-				phoneNumber = self._typemodel.get_value(itr, 0)
+			phoneNumber = self._get_number()
 		else:
 			phoneNumber = ""
+		if not phoneNumber:
+			self._action = self.ACTION_CANCEL
+
+		if self._action == self.ACTION_SEND_SMS:
+			smsMessage = self._smsDialog.run(phoneNumber, message)
+		else:
+			smsMessage = ""
+		if not smsMessage:
+			phoneNumber = ""
+			self._action = self.ACTION_CANCEL
 
 		self._typeviewselection.unselect_all()
+		self._typeview.remove_column(numberColumn)
+		self._typeview.remove_column(typeColumn)
+		self._typeview.set_model(None)
 		self._dialog.hide()
+		return self._action, phoneNumber, smsMessage
+
+	def _get_number(self):
+		model, itr = self._typeviewselection.get_selected()
+		if not itr:
+			return ""
+
+		phoneNumber = self._typemodel.get_value(itr, 0)
 		return phoneNumber
 
+	def _on_phonetype_dial(self, *args):
+		self._dialog.response(gtk.RESPONSE_OK)
+		self._action = self.ACTION_DIAL
+
+	def _on_phonetype_send_sms(self, *args):
+		self._dialog.response(gtk.RESPONSE_OK)
+		self._action = self.ACTION_SEND_SMS
+
 	def _on_phonetype_select(self, *args):
 		self._dialog.response(gtk.RESPONSE_OK)
+		self._action = self.ACTION_SELECT
 
 	def _on_phonetype_cancel(self, *args):
 		self._dialog.response(gtk.RESPONSE_CANCEL)
+		self._action = self.ACTION_CANCEL
+
+
+class SmsEntryDialog(object):
+
+	MAX_CHAR = 160
+
+	def __init__(self, widgetTree, gcBackend):
+		self._gcBackend = gcBackend
+		self._widgetTree = widgetTree
+		self._dialog = self._widgetTree.get_widget("smsDialog")
+
+		self._smsButton = self._widgetTree.get_widget("sendSmsButton")
+		self._smsButton.connect("clicked", self._on_send)
+
+		self._cancelButton = self._widgetTree.get_widget("cancelSmsButton")
+		self._cancelButton.connect("clicked", self._on_cancel)
+
+		self._letterCountLabel = self._widgetTree.get_widget("smsLetterCount")
+		self._message = self._widgetTree.get_widget("smsMessage")
+		self._smsEntry = self._widgetTree.get_widget("smsEntry")
+		self._smsEntry.get_buffer().connect("changed", self._on_entry_changed)
+
+	def run(self, number, message = ""):
+		if message:
+			self._message.set_markup(message)
+			self._message.show()
+		else:
+			self._message.set_markup("")
+			self._message.hide()
+		self._smsEntry.get_buffer().set_text("")
+		self._update_letter_count()
+
+		userResponse = self._dialog.run()
+		if userResponse == gtk.RESPONSE_OK:
+			entryBuffer = self._smsEntry.get_buffer()
+			enteredMessage = entryBuffer.get_text(entryBuffer.get_start_iter(), entryBuffer.get_end_iter())
+			enteredMessage = enteredMessage[0:self.MAX_CHAR]
+		else:
+			enteredMessage = ""
+
+		self._dialog.hide()
+		return enteredMessage
+
+	def _update_letter_count(self, *args):
+		entryLength = self._smsEntry.get_buffer().get_char_count()
+		charsLeft = self.MAX_CHAR - entryLength
+		self._letterCountLabel.set_text(str(charsLeft))
+		if charsLeft < 0:
+			self._smsButton.set_sensitive(False)
+		else:
+			self._smsButton.set_sensitive(True)
+
+	def _on_entry_changed(self, *args):
+		self._update_letter_count()
+
+	def _on_send(self, *args):
+		self._dialog.response(gtk.RESPONSE_OK)
+
+	def _on_cancel(self, *args):
+		self._dialog.response(gtk.RESPONSE_CANCEL)
 
 
 class Dialpad(object):
@@ -524,12 +644,15 @@ class RecentCallsView(object):
 		self._recentviewselection = None
 		self._onRecentviewRowActivatedId = 0
 
+		# @todo Make seperate columns for each item in recent item payload
 		textrenderer = gtk.CellRendererText()
 		self._recentviewColumn = gtk.TreeViewColumn("Calls")
 		self._recentviewColumn.pack_start(textrenderer, expand=True)
 		self._recentviewColumn.add_attribute(textrenderer, "text", 1)
 		self._recentviewColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
 
+		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
+
 	def enable(self):
 		assert self._backend.is_authed()
 		self._recentview.set_model(self._recentmodel)
@@ -545,7 +668,7 @@ class RecentCallsView(object):
 		self._recentview.remove_column(self._recentviewColumn)
 		self._recentview.set_model(None)
 
-	def number_selected(self, number):
+	def number_selected(self, action, number, message):
 		"""
 		@note Actual dial function is patched in later
 		"""
@@ -586,10 +709,109 @@ class RecentCallsView(object):
 		if not itr:
 			return
 
-		self.number_selected(self._recentmodel.get_value(itr, 0))
+		number = self._recentmodel.get_value(itr, 0)
+		number = make_ugly(number)
+		contactPhoneNumbers = [("Phone", number)]
+		description = self._recentmodel.get_value(itr, 1)
+
+		action, phoneNumber, message = self._phoneTypeSelector.run(contactPhoneNumbers, message = description)
+		if action == PhoneTypeSelector.ACTION_CANCEL:
+			return
+		assert phoneNumber
+
+		self.number_selected(action, phoneNumber, message)
 		self._recentviewselection.unselect_all()
 
 
+class MessagesView(object):
+
+	def __init__(self, widgetTree, backend, errorDisplay):
+		self._errorDisplay = errorDisplay
+		self._backend = backend
+
+		self._messagetime = 0.0
+		self._messagemodel = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
+		self._messageview = widgetTree.get_widget("messages_view")
+		self._messageviewselection = None
+		self._onMessageviewRowActivatedId = 0
+
+		textrenderer = gtk.CellRendererText()
+		# @todo Make seperate columns for each item in message payload
+		self._messageviewColumn = gtk.TreeViewColumn("Messages")
+		self._messageviewColumn.pack_start(textrenderer, expand=True)
+		self._messageviewColumn.add_attribute(textrenderer, "markup", 1)
+		self._messageviewColumn.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
+
+		self._phoneTypeSelector = PhoneTypeSelector(widgetTree, self._backend)
+
+	def enable(self):
+		assert self._backend.is_authed()
+		self._messageview.set_model(self._messagemodel)
+
+		self._messageview.append_column(self._messageviewColumn)
+		self._messageviewselection = self._messageview.get_selection()
+		self._messageviewselection.set_mode(gtk.SELECTION_SINGLE)
+
+		self._onMessageviewRowActivatedId = self._messageview.connect("row-activated", self._on_messageview_row_activated)
+
+	def disable(self):
+		self._messageview.disconnect(self._onMessageviewRowActivatedId)
+		self._messageview.remove_column(self._messageviewColumn)
+		self._messageview.set_model(None)
+
+	def number_selected(self, action, number, message):
+		"""
+		@note Actual dial function is patched in later
+		"""
+		raise NotImplementedError
+
+	def update(self):
+		if (time.time() - self._messagetime) < 300:
+			return
+		backgroundPopulate = threading.Thread(target=self._idly_populate_messageview)
+		backgroundPopulate.setDaemon(True)
+		backgroundPopulate.start()
+
+	def clear(self):
+		self._messagetime = 0.0
+		self._messagemodel.clear()
+
+	def _idly_populate_messageview(self):
+		self._messagetime = time.time()
+		self._messagemodel.clear()
+
+		try:
+			messageItems = self._backend.get_messages()
+		except RuntimeError, e:
+			self._errorDisplay.push_exception_with_lock(e)
+			self._messagetime = 0.0
+			messageItems = []
+
+		for header, number, relativeDate, message in messageItems:
+			number = make_ugly(number)
+			row = (number, message)
+			with gtk_toolbox.gtk_lock():
+				self._messagemodel.append(row)
+
+		return False
+
+	def _on_messageview_row_activated(self, treeview, path, view_column):
+		model, itr = self._messageviewselection.get_selected()
+		if not itr:
+			return
+
+		contactPhoneNumbers = [("Phone", self._messagemodel.get_value(itr, 0))]
+		description = self._messagemodel.get_value(itr, 1)
+
+		action, phoneNumber, message = self._phoneTypeSelector.run(contactPhoneNumbers, message = description)
+		if action == PhoneTypeSelector.ACTION_CANCEL:
+			return
+		assert phoneNumber
+
+		self.number_selected(action, phoneNumber, message)
+		self._messageviewselection.unselect_all()
+
+
 class ContactsView(object):
 
 	def __init__(self, widgetTree, backend, errorDisplay):
@@ -666,7 +888,7 @@ class ContactsView(object):
 		self._contactsview.set_model(None)
 		self._contactsview.remove_column(self._contactColumn)
 
-	def number_selected(self, number):
+	def number_selected(self, action, number, message):
 		"""
 		@note Actual dial function is patched in later
 		"""
@@ -747,22 +969,22 @@ class ContactsView(object):
 			return
 
 		contactId = self._contactsmodel.get_value(itr, 3)
+		contactName = self._contactsmodel.get_value(itr, 1)
 		try:
 			contactDetails = self._addressBook.get_contact_details(contactId)
 		except RuntimeError, e:
 			contactDetails = []
 			self._contactstime = 0.0
 			self._errorDisplay.push_exception(e)
-		contactDetails = [phoneNumber for phoneNumber in contactDetails]
+		contactPhoneNumbers = [phoneNumber for phoneNumber in contactDetails]
 
-		if len(contactDetails) == 0:
-			phoneNumber = ""
-		elif len(contactDetails) == 1:
-			phoneNumber = contactDetails[0][1]
-		else:
-			phoneNumber = self._phoneTypeSelector.run(contactDetails)
+		if len(contactPhoneNumbers) == 0:
+			return
 
-		if 0 < len(phoneNumber):
-			self.number_selected(phoneNumber)
+		action, phoneNumber, message = self._phoneTypeSelector.run(contactPhoneNumbers, message = contactName)
+		if action == PhoneTypeSelector.ACTION_CANCEL:
+			return
+		assert phoneNumber
 
+		self.number_selected(action, phoneNumber, message)
 		self._contactsviewselection.unselect_all()
